SelFlds.prg

* Program: SelFlds.prg
* Author:  Peter Volz
* Version: Clipper Summer '87
* Note(s)  Sample program to demonstrate ACreate().
*          Functions used: ACreate(), UDF_in(),
*                          UDF_out() [, CurDrv()]
*
* Copyright (c) 1988 Nantucket Corp.
*
dim1 = ADIR("*.dbf")         && How many .dbfs?
IF dim1 = 0                  && None?
   CLEAR SCREEN
   @ 2,24 SAY "Current directory: "+ CurDrv() + ":\" + CURDIR()
   @ 3,0 TO 3,79
   @ 20,0 TO 20,79
   @ 24,8 SAY "No database files in this directory. "+;
      "Press any key to exit..."
   INKEY(0)
ELSE                         && There are some .dbfs...
   DO WHILE LASTKEY() # 27
      CLEAR SCREEN
      @ 2,24 SAY "Current directory: "+CurDrv()+ ":\" +CURDIR()
      @ 3,0 TO 3,79
      @ 20,0 TO 20,79
      DECLARE dbfs[dim1]
      ADIR("*.dbf",dbfs)     && Fill array with .dbf names.

      *Display .dbf pop-up window.
      @ 24,30 SAY "Select a database ..."
      @ 4,31 TO 19,49 DOUBLE
      dbf_choice = ACHOICE(5, 33, 18, 47, dbfs)
      IF dbf_choice # 0      && A .dbf has been selected.
         CLEAR SCREEN
         choice_dbf = dbfs[dbf_choice]

         * Use selected DBF and load its field names into
         * an array.
         USE &choice_dbf.
         DECLARE fld_array[FCOUNT()]
         AFIELDS(fld_array)

         * Pass the array of field names to ACreate() and get
         * return value.
         new_array = ACREATE(fld_array)

         * If the array created by ACREATE() has defined elements,
         * then pass the array to DBEDIT() or ListIt().
         * For ListIt() see Nantucket News, Volume 3, Number 1.
         IF TYPE("new_array") = "C"
            CLEAR SCREEN
            @ 1,5 SAY "Database in use: "+ choice_dbf
            @ 3,0 TO 3,79
            @ 22,0 TO 22,79
            @ 23,10 SAY " Use cursor keys to navigate" +;
               " in the DBEDIT() window "
            @ 24,10 SAY "       <RETURN> or <ESC> " +;
                "to exit the window         "
            @ 3,0 TO 20,80 DOUBLE
            DBEDIT(4,1,19,78, &new_array.) 
            * Or ListIt(&new_array.).
         ENDIF
      ENDIF
      @ 21,0 CLEAR TO 24,79
      @ 22,0 TO 22,79
      @ 24,10 SAY "Any key to return to the DBF " +;
         "pop-up window, <ESC> to quit ..."
      INKEY(0)
   ENDDO
ENDIF
RETURN


* Function: ACreate()
* Note(s):  Parameters passed: An array of character strings.
*           Return value: Either a character string containing
*              the name of an array of character strings, or (if
*              this array contains no defined elements) a logical
*              false (.F.).
*
FUNCTION ACreate
PARAMETERS in_arr            && Incoming array.
PRIVATE  dim_in, init, rel, o_init, o_rel, cont
PRIVATE  kill_elem, keypress, width, halfwidth
PRIVATE  t, b, l1, r1, l2, r2

dim_in = len(in_arr)         && Dimension of incoming array.
PUBLIC out_arr[dim_in+1]     && Outgoing array declared as PUBLIC.
width = 0                    && Determine width of longest array
                             ** element.
FOR i = 1 TO dim_in
   width = MAX(width, LEN(in_arr[i]))
NEXT

width = MIN(width, 36)       && Largest width=36 (half a screen).
halfwidth = INT(width/2 + 0.5)
l1 = 20 - halfwidth          && Calculate ACHOICE() window
                             ** coordinates.
r1 = 19 + halfwidth
l2 = 60 - halfwidth
r2 = 59 + halfwidth
t  = 5
b  = 18

@ 2, 4 SAY "Choices available for selection"
@ 21,4 SAY "Use Cursor keys or first letter"
@ 22,4 SAY "        to move around         "
@ 23,4 SAY "<ESC> to confirm the selections"
@ 24,4 SAY "and leave the selection process"
@ 2,44 SAY "       Selected so far ...     "
@ 21,44 SAY "   <ENTER> to select an item   "
@ 22,44 SAY "   <DEL>   to remove an item   "
@ 23,44 SAY "   <Alt S> to SELECT ALL       "
@ 24,44 SAY "   <Alt R> to REMOVE ALL       "
@ 3,0 TO 3,79
@ 20,0 TO 20,79

init = 1       && Initial highlighted element in incoming array.
rel = 0        && Initial highlighted window position in
               ** incoming array.
o_init = 1     && Initial element in outgoing array.
o_rel = 0      && Initial window position in outgoing array.

@ t - 1, l1 - 2  TO  b + 1, r1 + 2  DOUBLE
@ t - 1, l2 - 2  TO  b + 1, r2 + 2  DOUBLE

cont = .T.
DO WHILE cont                && Loop while true (.T.).
   @ t,l2 CLEAR TO b,r2      && Clear right pop-up window area.
   KEYBOARD CHR(22)          && Dummy keystroke to bypass the
                             ** wait state for outgoing array.
                                
   * First ACHOICE() pops up outgoing array on right side
   * of screen.
   ACHOICE(t, l2, b, r2, out_arr, .T., "UDF_out", o_init, o_rel)

   * Second ACHOICE() pops up incoming array on left side
   * of screen.
   ACHOICE(t, l1, b, r1, in_arr, .T., "UDF_in", init, rel)
ENDDO
IF TYPE("out_arr[1]") = "U"  && If no defined element
   RETURN .F.                && in outgoing array,
                             ** return a logical type.
ELSE                         && Otherwise, return the name of
   RETURN "out_arr"          && the array as a char string.
ENDIF


* Function: UDF_in()
* Note(s):  User function to control response to keystrokes 
*           for ACHOICE(), which pops up the incoming array
*           of choices.
*
FUNCTION UDF_in
PARAMETERS mode, curr_elem, rel_pos
keypress = LASTKEY()
IF mode < 3                  && Idle, past top or past bottom.
   RETURN 2                  && Continue selection process.
ELSE                         && Mode = 3, key exception.
   DO CASE
   CASE keypress = 7         && DEL-key: scans out_arr for
                             ** a match to the current element
                             ** of in_arr, then deletes it
                             ** from out_arr.
      kill_elem = ASCAN(out_arr,in_arr[curr_elem])
      ADEL(out_arr, kill_elem)
      o_init = IF(o_init > 1, o_init - 1, o_init)
                             && Keep element in range.
      init = curr_elem       && Assign memvars for              
      rel = rel_pos          && next loop around.
      RETURN 0               && Leave ACHOICE(), loop around.

   CASE keypress = 13        && ENTER-key: copies current
                             && element of in_arr into next
                             && available element of out_arr.
      IF o_init <= dim_in
         out_arr[o_init] = in_arr[curr_elem]
         o_init = IF(o_init < dim_in, o_init + 1, o_init)
                             ** Keep element in range.
      ENDIF
      init = curr_elem + 1   && Assign memvars for
      rel = rel_pos + 1      && next loop around.
      RETURN 0               && Leave ACHOICE(), loop around.

   CASE keypress = 287       && Alt-S to SELECT ALL.
      ACOPY(in_arr, out_arr)
      init = curr_elem       && Assign memvars for
      rel = rel_pos          && next loop around.
      o_init = dim_in
      o_rel = MIN(dim_in, 14)  && max 14 elements in window.
      RETURN 0    
           
   CASE keypress = 275       && Alt-R to REMOVE ALL.
      FOR i = o_init TO 1 STEP -1
         ADEL(out_arr, i)
      NEXT
         init = curr_elem    && Assign memvars for
         rel = rel_pos       && next loop around.
         o_init = 1
         o_rel = 0
         RETURN 0

   CASE keypress = 27        && Escape key.
      cont = .F.             && Set flag to Exit DO WHILE loop.
      RETURN 0               && Leave ACHOICE(); goodbye.

   OTHERWISE
      RETURN 3               && First letter selects element.
   ENDCASE
ENDIF


* Function: UDF_out()
* Note(s):  User function for ACHOICE() which pops up the
*           outgoing array of selections made.
*
FUNCTION UDF_out
PARAMETERS outmode, out_elem, out_pos

* Assign memvars for next loop around.
o_init = IF(out_elem < dim_in, out_elem + 1, out_elem)
o_rel = out_pos + 1

RETURN 0                     && Leave ACHOICE() and proceed.
